<?php
/**
 * Plugin Name: WP Fastest Flusher
 * Plugin URI: https://null.null
 * Description: Flushes all cache from WP Fastest Cache weekly on Sundays.
 * Version: 1.0
 * Author: Andrzej Godziuk
 */

if (!defined('WPINC')) {
    die;
}

// Flush WP Fastest Cache
function flush_wpfc_cache_weekly() {
    if (function_exists('wpfc_clear_all_cache')) {
        wpfc_clear_all_cache();
    } elseif (function_exists('wp_fastest_cache_clear_all_cache')) {
        wp_fastest_cache_clear_all_cache();
    } else {
        error_log('WP Fastest Cache functions not found or plugin is inactive.');
    }
}

// Schedule the weekly cache flush
function schedule_flush_wpfc_cache() {
    if (!wp_next_scheduled('flush_wpfc_cache_weekly_event')) {
        wp_schedule_event(strtotime('next Sunday'), 'weekly', 'flush_wpfc_cache_weekly_event');
    }
}

// Hook into that event that will fire weekly
add_action('flush_wpfc_cache_weekly_event', 'flush_wpfc_cache_weekly');

// Activation hook
function on_activation_flush_wpfc_cache() {
    schedule_flush_wpfc_cache();
}

// Deactivation hook
function on_deactivation_flush_wpfc_cache() {
    $timestamp = wp_next_scheduled('flush_wpfc_cache_weekly_event');
    wp_unschedule_event($timestamp, 'flush_wpfc_cache_weekly_event');
}

register_activation_hook(__FILE__, 'on_activation_flush_wpfc_cache');
register_deactivation_hook(__FILE__, 'on_deactivation_flush_wpfc_cache');

/*
 * 2nd functionality of this plugin: notify if WP-Cron health is bad
 */


// Hook into 'shutdown' action which runs after the page is generated, but before PHP stops execution.
add_action('shutdown', 'wp_cron_health_checker_run', 999);

function wp_cron_health_checker_run() {
    // Ensure this runs only on front-end
    if (is_admin()) return;

    $last_run = get_option('wp_cron_health_checker_last_run');
    $current_time = time();

    // Check if it's the first run or more than 24 hours since last run
    if (!$last_run || ($current_time - $last_run) > 86400) {
        // Update last run time
        update_option('wp_cron_health_checker_last_run', $current_time);

        // Check WP-Cron health
        $overdue_tasks = wp_cron_health_checker_check_health();
        if (!empty($overdue_tasks)) {
            // WP-Cron health is bad, send email
            wp_cron_health_checker_send_email($overdue_tasks);
        }
    }
}

function wp_cron_health_checker_check_health() {
    $scheduledTasks = _get_cron_array();
    //if (!$scheduledTasks) return false; // Indicates bad health if no tasks are scheduled

    $current_time = time();
    $critical_tasks = ['wp_version_check', 'wp_update_plugins', 'wp_update_themes', 'my_custom_backup_task']; // Add your critical scheduled tasks here
    $overdue_tasks = [];
    $missed_critical_tasks = false;

    foreach ($scheduledTasks as $timestamp => $tasks) {
        if ($timestamp < $current_time) {
            // Task is overdue, check if it's critical
            foreach ($tasks as $hook => $task) {
                if (in_array($hook, $critical_tasks)) {
                    $missed_critical_tasks = true;
                }
                $overdue_tasks[] = $hook; // Collecting overdue task for diagnostic data
            }
        }
    }

    // If there are overdue tasks or missed critical tasks, consider WP-Cron health to be bad
    if (!empty($overdue_tasks) || $missed_critical_tasks) {
        return $overdue_tasks; // Bad health
    }

    return []; // Good health
}

function wp_cron_health_checker_send_email($overdue_tasks) {
    if(empty($overdue_tasks)) return;
    $site_url = get_site_url();
    $url_components = parse_url($site_url);
    $domain = str_replace('www.', '', $url_components['host']);

    $to = 'wp@massivescale.net';
    $subject = "WP-Cron Health Issue Detected on {$domain}";
    $body = "WP-Cron health issue detected at {$site_url}.\n\n";
    
    if (!empty($overdue_tasks)) {
        $body .= "The following tasks are overdue: " . implode(', ', $overdue_tasks) . "\n\n";
    }
    if($overdue_tasks === false) {
        $body .= "No scheduled tasks - this indicates bad health";
    }
    
    $body .= "Please check the WP-Cron settings and health.\n\n";
    $body .= "lxsh {$domain}";
    $headers = array('Content-Type: text/plain; charset=UTF-8', 'Date: ' . date('r', $_SERVER['REQUEST_TIME']));
    
    wp_mail($to, $subject, $body, $headers);
}

if ( defined( 'WP_CLI' ) && WP_CLI ) {
    /**
     * Prints all overdue tasks.
     *
     * ## EXAMPLES
     *
     *     wp cron health-check
     */
    function wp_cli_cron_health_check() {
        $overdue_tasks = wp_cron_health_checker_check_health();

        if ( is_array( $overdue_tasks ) && !empty( $overdue_tasks ) ) {
            WP_CLI::success( "Overdue tasks found:" );
            foreach ( $overdue_tasks as $task ) {
                WP_CLI::line( "- " . $task );
            }
        } else if ( $overdue_tasks === true ) {
            WP_CLI::success( "No overdue tasks. WP-Cron health is good." );
        } else {
            WP_CLI::error( "Unable to check WP-Cron health." );
        }
    }

    WP_CLI::add_command( 'cron health-check', 'wp_cli_cron_health_check' );
}
